@@ -23,7 +23,7 @@
 /* Dispose of the command structure passed. */
 void
 dispose_command (command)
-     register COMMAND *command;
+     COMMAND *command;
 {
   if (!command) return;
 
@@ -41,6 +41,18 @@ dispose_command (command)
 	free (c);
 	break;
       }
+
+#if defined (SELECT_COMMAND)
+    case cm_select:
+      {
+	register SELECT_COM *c = command->value.Select;
+	dispose_word (c->name);
+	dispose_words (c->map_list);
+	dispose_command (c->action);
+	free (c);
+	break;
+      }
+#endif
     
     case cm_group:
       {
