@@ -1,16 +1,35 @@
 /* shell.h -- The data structures used by the shell */
 
+/* Copyright (C) 1993 Free Software Foundation, Inc.
+
+   This file is part of GNU Bash, the Bourne Again SHell.
+
+   Bash is free software; you can redistribute it and/or modify it under
+   the terms of the GNU General Public License as published by the Free
+   Software Foundation; either version 2, or (at your option) any later
+   version.
+
+   Bash is distributed in the hope that it will be useful, but WITHOUT ANY
+   WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+   for more details.
+
+   You should have received a copy of the GNU General Public License along
+   with Bash; see the file COPYING.  If not, write to the Free Software
+   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+
 #include "config.h"
+#include "command.h"
 #include "general.h"
 #include "error.h"
 #include "variables.h"
 #include "quit.h"
 #include "maxpath.h"
 #include "unwind_prot.h"
-#include "command.h"
 #include "dispose_cmd.h"
 #include "make_cmd.h"
 #include "subst.h"
+#include "externs.h"
 
 extern int EOF_Reached;
 
@@ -33,6 +52,9 @@ extern jmp_buf top_level, catch;
 #define EXECUTION_FAILURE 1
 #define EXECUTION_SUCCESS 0
 
+/* Usage messages by builtins result in a return status of 2. */
+#define EX_USAGE 2
+
 /* Special exit status used when the shell is asked to execute a
    binary file as a shell script. */
 #define EX_BINARY_FILE 126
